﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using com.citizen.sdk.LabelPrint;
using System.Diagnostics;

namespace LabelApplication
{
    public partial class Form1 : Form
    {
        private enum eErrorKind
        {
            eErrConnect = 1,
            eErrPrinterCheck,
            eErrPrint,
            eErrPrinterStatus
        };


        public Form1()
        {
            InitializeComponent();

            // ComboBoxColumnで使用するDataTableの作成
            DataTable dataTable1 = new DataTable("ComboBox");
            dataTable1.Columns.Add("Display", typeof(string));
            dataTable1.Columns.Add("Value", typeof(int));
            dataTable1.Rows.Add("NET", LabelConst.CLS_PORT_NET);
            dataTable1.Rows.Add("USB", LabelConst.CLS_PORT_USB);
            dataTable1.Rows.Add("COM", LabelConst.CLS_PORT_COM);
            dataTable1.Rows.Add("LPT", LabelConst.CLS_PORT_LPT);
            dataTable1.Rows.Add("Bluetooth", LabelConst.CLS_PORT_Bluetooth);
            ifCmbBox.DataSource = dataTable1;
            ifCmbBox.DisplayMember = "Display";
            ifCmbBox.ValueMember = "Value";
            ifCmbBox.SelectedIndex = 0;
        }


// Event Handler ------------------------------------------------------------- //

        private void Form1_Load(object sender, EventArgs e)
        {
            listView1.Items.Clear();
            listView1.Columns.Add("Port", 120, HorizontalAlignment.Left);
            listView1.Columns.Add("", 210, HorizontalAlignment.Left);
        }


        private void ifCmbBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            listView1.Items.Clear();
        }


        private void searchBtn_Click(object sender, EventArgs e)
        {
            int searchmode = 0;     // searchmode 0 : SearchCitizenPrinter
                                    //            1 : SearchLabelPrinter
            int res = 0;
            int type = 0;
            LabelPrinter printer = new LabelPrinter();

            //Initialize
            listView1.Items.Clear();
            type = (int)ifCmbBox.SelectedValue;

            if (searchmode == 0)
            {
                // 
                // example for SearchCitizenPrinter
                //

                CitizenPrinterInfo[] info;

                //Search available Citizen printers
                info = printer.SearchCitizenPrinter(type, 10, out res);
                if (res != LabelConst.CLS_SUCCESS)
                {
                    if (res == LabelConst.CLS_E_NO_LIST)
                    {
                        MessageBox.Show("Printer Not Found.", "Warning");
                    }
                    else
                    {
                        MessageBox.Show("SearchCitizenPrinter Failed", "Error");
                    }
                    return;
                }

                //List up all available Citizen printers in the listView
                for (int i = 0; i < info.Length; i++)
                {
                    string[] CLSs = new string[2];
                    switch (type)
                    {

                        case LabelConst.CLS_PORT_NET:
                            CLSs[0] = info[i].ipAddress;
                            CLSs[1] = info[i].macAddress;
                            break;

                        case LabelConst.CLS_PORT_USB:
                            CLSs[0] = info[i].deviceName;
                            CLSs[1] = info[i].printerModel;
                            break;

                        case LabelConst.CLS_PORT_COM:
                            CLSs[0] = info[i].deviceName;
                            break;

                        case LabelConst.CLS_PORT_LPT:
                            CLSs[0] = info[i].deviceName;
                            break;

                        case LabelConst.CLS_PORT_Bluetooth:
                            CLSs[0] = info[i].bdAddress;
                            CLSs[1] = info[i].deviceName;
                            break;

                    }
                    listView1.Items.Add(new ListViewItem(CLSs));
                }
            }
            else
            {
                // 
                // example for SearchLabelPrinter
                //

                string[] address;

                //Search available Label printers
                address = printer.SearchLabelPrinter(type, 10, out res);
                if (res != LabelConst.CLS_SUCCESS)
                {
                    if (res == LabelConst.CLS_E_NO_LIST)
                    {
                        MessageBox.Show("Printer Not Found.", "Warning");
                    }
                    else
                    {
                        MessageBox.Show("SearchLabelPrinter Failed", "Error");
                    }
                    return;
                }

                //List up all available Label printers in the listView
                for (int i = 0; i < address.Length; i++)
                {
                    string CLSs = address[i];
                    listView1.Items.Add(new ListViewItem(CLSs));
                }
            }

            listView1.Focus();
            listView1.Items[0].Selected = true;

        }


        // Example1
        //  Label Print (Single)
        private void button1_Click(object sender, EventArgs e)
        {
            // Check if a printer is selected
            if (listView1.SelectedItems.Count <= 0)
            {
                MessageBox.Show("No printer is selected.", "Error");
                return;
            }

            //Get I/F type
            int type = (int)ifCmbBox.SelectedValue;

            // Create an instance (LabelPrinter Class)
            LabelPrinter printer = new LabelPrinter();

            // Connect
            int ret = printer.Connect(type, listView1.SelectedItems[0].Text);
            if (ret != LabelConst.CLS_SUCCESS)
            {
                MessageBox.Show("Connect error: " + ret.ToString(), "Error");
                return;
            }

            // Set Property (Measurement unit)
            printer.SetMeasurementUnit(LabelConst.CLS_UNIT_INCH);

            // Create an instance (LabelDesign Class)
            LabelDesign design = new LabelDesign();

            // Design label
            this.DesignLabel_Pattern1(design);

            // Print label
            printer.Print(design, 1);

            // Disconnect
            printer.Disconnect();
        }


        // Example2
        //  Label Print (Multi:TearOff and AutoCut)
        private void button2_Click(object sender, EventArgs e)
        {
            // Check if a printer is selected
            if (listView1.SelectedItems.Count <= 0)
            {
                MessageBox.Show("No printer is selected.", "Error");
                return;
            }

            int quantity = 3;
            int serialNo = 0;
            int type = (int)ifCmbBox.SelectedValue;

            // Create an instance (LabelPrinter Class)
            LabelPrinter printer = new LabelPrinter();

            // Connect
            int ret = printer.Connect(type, listView1.SelectedItems[0].Text);
            if (ret != LabelConst.CLS_SUCCESS)
            {
                MessageBox.Show("Connect error: " + ret.ToString(), "Error");
            }
 
            // Set Property (Measurement unit)
            printer.SetMeasurementUnit(LabelConst.CLS_UNIT_INCH);

            //---- Print Design2 ----//
            for (int loopCnt = 1; loopCnt <= quantity; loopCnt++)
            {
                // Create an instance (LabelDesign Class)
                LabelDesign design = new LabelDesign();

                // Design label
                this.DesignLabel_Pattern2(design, serialNo);

                if (loopCnt < quantity)
                {
                    // Set Property (Tear Off)
                    printer.SetMediaHandling(LabelConst.CLS_MEDIAHANDLING_TEAROFF);

                    // Print design
                    printer.Print(design, 1);
                }
                else
                {
                    // Set Property (Cut on last page)
                    printer.SetMediaHandling(LabelConst.CLS_MEDIAHANDLING_CUT);

                    // Print label
                    printer.Print(design, 1);
                }

                // Count up serial number
                serialNo++;
            }

            // Disconnect
            printer.Disconnect();
        }


        // Example3
        //  Print and check the printer status
        private void button3_Click(object sender, EventArgs e)
        {
            String sStsMsg = "";
            int iPtrErr = 0;
            int iResult = LabelConst.CLS_SUCCESS;
            int type = (int)ifCmbBox.SelectedValue;

            // Check if a printer is selected
            if (listView1.SelectedItems.Count <= 0)
            {
                MessageBox.Show("No printer is selected.", "Error");
                return;
            }

            // Create an instance (LabelPrinter Class)
            LabelPrinter printer = new LabelPrinter();

            // Connect
            iResult = printer.Connect(type, listView1.SelectedItems[0].Text);
            if (iResult != LabelConst.CLS_SUCCESS)
            {
                this.PrinterErrorProc(printer, eErrorKind.eErrConnect, iResult, null);
                return;
            }

            // Get the printer status
            iResult = printer.PrinterCheck();
            if (iResult != LabelConst.CLS_SUCCESS)
            {
                this.PrinterErrorProc(printer, eErrorKind.eErrPrinterCheck, iResult, null);
                printer.Disconnect();       // Disconnect
                return;
            }

            // Checks the status and populates into each property
            sStsMsg = "";
            this.CheckPrinterStatus(printer, ref iPtrErr, ref sStsMsg);
            if (iPtrErr != 0)
            {
                this.PrinterErrorProc(printer, eErrorKind.eErrPrinterStatus, 0, sStsMsg);
                printer.Disconnect();       // Disconnect
                return;
            }

            // Set Property (Measurement unit)
            printer.SetMeasurementUnit(LabelConst.CLS_UNIT_INCH);

            // Create an instance (LabelDesign Class)
            LabelDesign design = new LabelDesign();

            // Design label
            this.DesignLabel_Pattern3(design);

            // Print label
            iResult = printer.Print(design, 2);
            if (iResult != LabelConst.CLS_SUCCESS)
            {
                this.PrinterErrorProc(printer, eErrorKind.eErrPrint, iResult, null);
                printer.Disconnect();           // Disconnect
                return;
            }

            System.Threading.Thread.Sleep(500);

            // Check printing
            while (true)
            {
                // Get the printer status
                iResult = printer.PrinterCheck();
                if (iResult != LabelConst.CLS_SUCCESS)
                {
                    this.PrinterErrorProc(printer, eErrorKind.eErrPrinterCheck, iResult, null);
                    printer.Disconnect();       // Disconnect
                    return;
                }

                // Checks the Printing and BatchProcessing populates
                //Debug.WriteLine("Printing:" + printer.GetPrinting() + ", BatchProcessing:" + printer.GetBatchProcessing());
                if (printer.GetPrinting() == 0 && printer.GetBatchProcessing() == 0)
                {
                    break;
                }
            }

            // Checks the error status and populates into each property
            sStsMsg = "";
            this.CheckPrinterStatus(printer, ref iPtrErr, ref sStsMsg);
            if (iPtrErr != 0)
            {
                this.PrinterErrorProc(printer, eErrorKind.eErrPrinterStatus, 0, sStsMsg);
            }
            else
            {
                MessageBox.Show("Success.", "Print Result");
            }

            // Disconnect
            printer.Disconnect();

        }


// Label design ------------------------------------------------------------- //

        private void DesignLabel_Pattern1(LabelDesign design)
        {

            this.mediumFont(design, "12 LBS     1 OF 2", 230, 560);

            this.smallFont(design, "CITIZEN SYSTEMS AMERICA", 10, 565);
            this.smallFont(design, "1234567890", 10, 555);
            this.smallFont(design, "STE 404", 10, 545);
            this.smallFont(design, "363 VAN NESS WAY", 10, 535);
            this.smallFont(design, "TORRANCE  CA 90501", 10, 525);

            this.mediumBoldFont(design, "SHIP", 15, 500);
            this.mediumBoldFont(design, "TO:", 19, 472);

            this.mediumFont(design, "LYMAN C SMITH", 60, 500);
            this.mediumFont(design, "9876543210", 60, 482);
            this.mediumFont(design, "SMITH TOWER", 60, 464);
            this.mediumFont(design, "STE 220", 60, 446);
            this.mediumFont(design, "506 2ND AVE", 60, 428);

            this.mediumBoldFont(design, "SEATTLE WA 98104-2307", 60, 407);

            design.DrawLine(0, 405, 450, 405, 1);
            design.DrawLine(122, 404, 122, 290, 1);
            design.DrawLine(0, 284, 450, 284, 10);

            //CANADA: string[] maxicode = { "M2N0A9", "068", "001", "1Z123X560120754868" };
            string[] maxicode = { "98104", "2307", "840", "001", "1Z123X560120754868" };
            design.DrawMaxiCode(maxicode, LabelConst.CLS_RT_NORMAL, 10, 298);

            design.DrawBarCode("&D420981042307", LabelConst.CLS_BCS_CODE128, LabelConst.CLS_RT_NORMAL, 6, 3, 050, 180, 320, LabelConst.CLS_BCS_TEXT_HIDE);
            this.mediumFont(design, "(420) 98104-2307", 200, 295);

            this.smallFont(design, "(420) SHIP TO POSTAL CODE", 140, 385);

            design.DrawTextPCFont("UPS NEXT DAY AIR", "Arial", LabelConst.CLS_RT_NORMAL, 100, 100, 20, LabelConst.CLS_FNT_BOLD, 10, 245);
            this.mediumSmallFont(design, "TRACKING #: 1Z 123 X56 01 2075 4868", 10, 230);
            design.DrawTextPCFont("1", "Arial", LabelConst.CLS_RT_NORMAL, 100, 100, 48, LabelConst.CLS_FNT_BOLD, 310, 215);

            design.DrawLine(0, 225, 450, 225, 1);
            design.DrawBarCode("1Z123X560120754868", LabelConst.CLS_BCS_CODE128, LabelConst.CLS_RT_NORMAL, 6, 3, 100, 30, 105, LabelConst.CLS_BCS_TEXT_HIDE);
            design.DrawLine(0, 84, 450, 84, 10);

        }


        private void DesignLabel_Pattern2(LabelDesign design, int serialNo)
        {

            design.DrawTextPtrFont("Product Name: A1B23CD456", LabelConst.CLS_LOCALE_JP, LabelConst.CLS_PRT_FNT_TRIUMVIRATE, LabelConst.CLS_RT_NORMAL, 1, 1, LabelConst.CLS_PRT_FNT_SIZE_14, 10, 215);
            design.DrawBarCode("A1B23CD456", LabelConst.CLS_BCS_CODE128, LabelConst.CLS_RT_NORMAL, 6, 3, 60, 10, 10, LabelConst.CLS_BCS_TEXT_HIDE);

            String sSerialNo = "Serial No: ";
            sSerialNo += serialNo.ToString("D10");
            design.DrawTextPtrFont(sSerialNo, LabelConst.CLS_LOCALE_JP, LabelConst.CLS_PRT_FNT_TRIUMVIRATE, LabelConst.CLS_RT_NORMAL, 1, 1, LabelConst.CLS_PRT_FNT_SIZE_14, 10, 75);
            design.DrawBarCode(sSerialNo, LabelConst.CLS_BCS_CODE128, LabelConst.CLS_RT_NORMAL, 6, 3, 60, 10, 150, LabelConst.CLS_BCS_TEXT_HIDE);

        }


        private void DesignLabel_Pattern3(LabelDesign design)
        {
            // Order No
            design.DrawTextPtrFont("Order No:", LabelConst.CLS_LOCALE_JP, LabelConst.CLS_PRT_FNT_TRIUMVIRATE, LabelConst.CLS_RT_RIGHT90, 1, 1, LabelConst.CLS_PRT_FNT_SIZE_14, 360, 280);

            // Date
            design.DrawTextPtrFont("Date:", LabelConst.CLS_LOCALE_JP, LabelConst.CLS_PRT_FNT_TRIUMVIRATE, LabelConst.CLS_RT_RIGHT90, 1, 1, LabelConst.CLS_PRT_FNT_SIZE_14, 320, 280);

            // To:
            design.DrawTextPtrFont("To:", LabelConst.CLS_LOCALE_JP, LabelConst.CLS_PRT_FNT_TRIUMVIRATE, LabelConst.CLS_RT_RIGHT90, 1, 1, LabelConst.CLS_PRT_FNT_SIZE_14, 280, 280);

            // From:
            design.DrawTextPtrFont("From:", LabelConst.CLS_LOCALE_JP, LabelConst.CLS_PRT_FNT_TRIUMVIRATE, LabelConst.CLS_RT_RIGHT90, 1, 1, LabelConst.CLS_PRT_FNT_SIZE_14, 240, 280);

            // Rectangle
            design.DrawRect(230, 5, 155, 280, 1);


            // Order No
            design.DrawTextPtrFont("Order No:", LabelConst.CLS_LOCALE_JP, LabelConst.CLS_PRT_FNT_TRIUMVIRATE, LabelConst.CLS_RT_RIGHT90, 1, 1, LabelConst.CLS_PRT_FNT_SIZE_14, 140, 280);

            // Date
            design.DrawTextPtrFont("Date:", LabelConst.CLS_LOCALE_JP, LabelConst.CLS_PRT_FNT_TRIUMVIRATE, LabelConst.CLS_RT_RIGHT90, 1, 1, LabelConst.CLS_PRT_FNT_SIZE_14, 100, 280);

            // To:
            design.DrawTextPtrFont("To:", LabelConst.CLS_LOCALE_JP, LabelConst.CLS_PRT_FNT_TRIUMVIRATE, LabelConst.CLS_RT_RIGHT90, 1, 1, LabelConst.CLS_PRT_FNT_SIZE_14, 60, 280);

            // From:
            design.DrawTextPtrFont("From:", LabelConst.CLS_LOCALE_JP, LabelConst.CLS_PRT_FNT_TRIUMVIRATE, LabelConst.CLS_RT_RIGHT90, 1, 1, LabelConst.CLS_PRT_FNT_SIZE_14, 20, 280);

            // Rectangle
            design.DrawRect(10, 5, 155, 280, 1);

        }


        private void smallFont(LabelDesign design, String data, int x, int y)
        {
            design.DrawTextPtrFont(data,                                //string data
                                   LabelConst.CLS_LOCALE_OTHER,         //int locale
                                   LabelConst.CLS_PRT_FNT_TRIUMVIRATE,  //int font
                                   LabelConst.CLS_RT_NORMAL,            //int rotation
                                   1,                                   //int hexp
                                   1,                                   //int vexp
                                   LabelConst.CLS_PRT_FNT_SIZE_8,       //int size
                                   x,                                   //int x
                                   y);                                  //int y

        }


        private void mediumSmallFont(LabelDesign design, String data, int x, int y)
        {
            design.DrawTextPtrFont(data,                                //string data
                                   LabelConst.CLS_LOCALE_OTHER,         //int locale
                                   LabelConst.CLS_PRT_FNT_TRIUMVIRATE,  //int font
                                   LabelConst.CLS_RT_NORMAL,            //int rotation
                                   1,                                   //int hexp
                                   1,                                   //int vexp
                                   LabelConst.CLS_PRT_FNT_SIZE_10,      //int size
                                   x,                                   //int x
                                   y);                                  //int y
        }


        private void mediumFont(LabelDesign design, String data, int x, int y)
        {
            design.DrawTextPtrFont(data,                                //string data
                                   LabelConst.CLS_LOCALE_OTHER,         //int locale
                                   LabelConst.CLS_PRT_FNT_TRIUMVIRATE,  //int font
                                   LabelConst.CLS_RT_NORMAL,            //int rotation
                                   1,                                   //int hexp
                                   1,                                   //int vexp
                                   LabelConst.CLS_PRT_FNT_SIZE_12,      //int size
                                   x,                                   //int x
                                   y);                                  //int y
        }


        private void mediumBoldFont(LabelDesign design, String data, int x, int y)
        {
            design.DrawTextPtrFont(data,                                //string data
                                   LabelConst.CLS_LOCALE_OTHER,         //int locale
                                   LabelConst.CLS_PRT_FNT_TRIUMVIRATE_B,//int font
                                   LabelConst.CLS_RT_NORMAL,            //int rotation
                                   1,                                   //int hexp
                                   1,                                   //int vexp
                                   LabelConst.CLS_PRT_FNT_SIZE_14,      //int size
                                   x,                                   //int x
                                   y);                                  //int y
        }


// Other method ------------------------------------------------------------- //

        private void CheckPrinterStatus(LabelPrinter printer, ref int error, ref String message)
        {
            int iStatus = 0;


            // Check printer status
            {
                // Make error message
                message += "Printer Status\r\n";

                iStatus = printer.GetCommandInterpreterInAction();
                message += " " + iStatus.ToString() + ": Command interpreter in action" + "\r\n";
                error += iStatus;

                iStatus = printer.GetPaperError();
                message += " " + iStatus.ToString() + ": Paper error" + "\r\n";
                error += iStatus;

                iStatus = printer.GetRibbonEnd();
                message += " " + iStatus.ToString() + ": Ribbon end" + "\r\n";
                error += iStatus;

                iStatus = printer.GetBatchProcessing();
                message += " " + iStatus.ToString() + ": Batch processing" + "\r\n";
                error += iStatus;

                iStatus = printer.GetPrinting();
                message += " " + iStatus.ToString() + ": Printing" + "\r\n";
                error += iStatus;

                iStatus = printer.GetPause();
                message += " " + iStatus.ToString() + ": Pause" + "\r\n";
                error += iStatus;

                iStatus = printer.GetWaitingForPeeling();
                message += " " + iStatus.ToString() + ": Waiting for peeling" + "\r\n";
                error += iStatus;

                iStatus = printer.GetPrintHeadLowTemp();
                message += " " + iStatus.ToString() + ": Print head low temp" + "\r\n";
                error += iStatus;

                iStatus = printer.GetPrintHeadFailure();
                message += " " + iStatus.ToString() + ": Print head failure" + "\r\n";
                error += iStatus;

                iStatus = printer.GetPrintHeadOverheat();
                message += " " + iStatus.ToString() + ": Print head overheat" + "\r\n";
                error += iStatus;

                iStatus = printer.GetMechanismOpen();
                message += " " + iStatus.ToString() + ": Mechanism open" + "\r\n";
                error += iStatus;

                iStatus = printer.GetAutoCutterError();
                message += " " + iStatus.ToString() + ": Auto cutter error" + "\r\n";
                error += iStatus;

                iStatus = printer.GetFanMotorError();
                message += " " + iStatus.ToString() + ": Fan motor error" + "\r\n";
                error += iStatus;

                iStatus = printer.GetMiscError();
                message += " " + iStatus.ToString() + ": Misc error" + "\r\n";
                error += iStatus;
            }

        }


        private void PrinterErrorProc(LabelPrinter printer, eErrorKind eError, int iErrCode, String sMsg)
        {
            String sTmpMsg = "";

            if (eError == eErrorKind.eErrConnect)
            {
                MessageBox.Show("Connect failure: " + iErrCode.ToString(), "Error");
            }
            else if (eError == eErrorKind.eErrPrinterCheck)
            {
                MessageBox.Show("PrinterCheck failure: " + iErrCode.ToString(), "Error");
            }
            else if (eError == eErrorKind.eErrPrint)
            {
                MessageBox.Show("Print failure: " + iErrCode.ToString(), "Error");
            }
            else if (eError == eErrorKind.eErrPrinterStatus)
            {
                // Show error dialog
                DialogResult dlgRet;
                sTmpMsg = "The printer can not print.\r\n" + "Select 'Yes' to reset the printer, or 'No' if you want to restore it manually.\r\n" + "*Clear print buffer is select 'Yes'. \r\n\r\n" + sMsg;
                dlgRet = MessageBox.Show(sTmpMsg, "Printer Error", MessageBoxButtons.YesNo);

                // Reset the print buffer
                if (dlgRet == DialogResult.Yes)
                {
                    printer.ClearOutput();
                }
            }
            else
            {
                // No Operation
            }
        }


    }
}
